@@ -198,9 +198,29 @@ function HandleFolderSync($backend, $protocolversion) {
     // longer required
     $syncstate = $statemachine->getSyncState($synckey);
 
+    // additional information about already seen folders
+    $sfolderstate = $statemachine->getSyncState("s".$synckey);
+    
+    if (!$sfolderstate) {
+        $foldercache = array();
+        if ($sfolderstate === false)
+            debugLog("Error: FolderChacheState for state 's". $synckey ."' not found. Reinitializing...");
+    }
+    else {
+    	$foldercache = unserialize($sfolderstate);
+
+    	// transform old seenfolder array
+    	if (array_key_exists("0", $foldercache)) {
+    		$tmp = array();
+    		foreach($foldercache as $s) $tmp[$s] = new SyncFolder();
+    		$foldercache = $tmp;
+    	}
+    }
+        
     // We will be saving the sync state under 'newsynckey'
     $newsynckey = $statemachine->getNewSyncKey($synckey);
-
+    $changes = false;
+    
     if($decoder->getElementStartTag(SYNC_FOLDERHIERARCHY_CHANGES)) {
         // Ignore <Count> if present
         if($decoder->getElementStartTag(SYNC_FOLDERHIERARCHY_COUNT)) {
@@ -229,9 +249,16 @@ function HandleFolderSync($backend, $protocolversion) {
                 case SYNC_ADD:
                 case SYNC_MODIFY:
                     $serverid = $importer->ImportFolderChange($folder);
+                    // add folder to the serverflags
+                    $foldercache[$serverid] = $folder;
+                    $changes = true;
                     break;
                 case SYNC_REMOVE:
                     $serverid = $importer->ImportFolderDeletion($folder);
+                    $changes = true;
+                    // remove folder from the folderchache
+                    if (array_key_exists($serverid, $foldercache)) 
+                        unset($foldercache[$serverid]);
                     break;
             }
 
@@ -253,7 +280,7 @@ function HandleFolderSync($backend, $protocolversion) {
     // before sending the actual data. As the amount of data done in this operation
     // is rather low, this is not memory problem. Note that this is not done when
     // sync'ing messages - we let the exporter write directly to WBXML.
-    $importer = new ImportHierarchyChangesMem($encoder);
+    $importer = new ImportHierarchyChangesMem($foldercache);
 
     // Request changes from backend, they will be sent to the MemImporter passed as the first
     // argument, which stores them in $importer. Returns the new sync state for this exporter.
@@ -273,7 +300,8 @@ function HandleFolderSync($backend, $protocolversion) {
         $encoder->endTag();
 
         $encoder->startTag(SYNC_FOLDERHIERARCHY_SYNCKEY);
-        $encoder->content($newsynckey);
+        // only send new synckey if changes were processed or there are outgoing changes
+        $encoder->content((($changes || $importer->count > 0)?$newsynckey:$synckey));
         $encoder->endTag();
 
         $encoder->startTag(SYNC_FOLDERHIERARCHY_CHANGES);
@@ -284,7 +312,13 @@ function HandleFolderSync($backend, $protocolversion) {
 
             if(count($importer->changed) > 0) {
                 foreach($importer->changed as $folder) {
-                    $encoder->startTag(SYNC_FOLDERHIERARCHY_ADD);
+                	// send a modify flag if the folder is already known on the device
+                	if (isset($folder->serverid) && array_key_exists($folder->serverid, $foldercache) !== false)
+                        $encoder->startTag(SYNC_FOLDERHIERARCHY_UPDATE);
+                	else 
+                        $encoder->startTag(SYNC_FOLDERHIERARCHY_ADD);
+                    $foldercache[$folder->serverid] = $folder;
+                    
                     $folder->encode($encoder);
                     $encoder->endTag();
                 }
@@ -297,6 +331,10 @@ function HandleFolderSync($backend, $protocolversion) {
                             $encoder->content($folder);
                         $encoder->endTag();
                     $encoder->endTag();
+
+                    // remove folder from the folderchache
+                    if (array_key_exists($folder, $foldercache)) 
+                        unset($foldercache[$folder]);
                 }
             }
         }
@@ -307,6 +345,7 @@ function HandleFolderSync($backend, $protocolversion) {
     // Save the sync state for the next time
     $syncstate = $exporter->GetState();
     $statemachine->setSyncState($newsynckey, $syncstate);
+    $statemachine->setSyncState("s".$newsynckey, serialize($foldercache));
 
 
     return true;
@@ -571,8 +610,20 @@ function HandleSync($backend, $protocolversion, $devid) {
         $encoder->startTag(SYNC_FOLDERS);
         {
             foreach($collections as $collection) {
-                // Get a new sync key to output to the client if any changes have been requested or have been sent
-                if (isset($collection["importedchanges"]) || isset($collection["getchanges"]) || $collection["synckey"] == "0")
+            	// initialize exporter to get changecount
+            	$changecount = 0;
+            	if(isset($collection["getchanges"])) {
+                    // Use the state from the importer, as changes may have already happened
+                    $exporter = $backend->GetExporter($collection["collectionid"]);
+
+                    $filtertype = isset($collection["filtertype"]) ? $collection["filtertype"] : false;
+                    $exporter->Config($importer, $collection["class"], $filtertype, $collection["syncstate"], 0, $collection["truncation"]);
+
+                    $changecount = $exporter->GetChangeCount();
+            	}
+            	
+                // Get a new sync key to output to the client if any changes have been requested or will be send
+                if (isset($collection["importedchanges"]) || $changecount > 0 || $collection["synckey"] == "0")
                     $collection["newsynckey"] = $statemachine->getNewSyncKey($collection["synckey"]);
 
                 $encoder->startTag(SYNC_FOLDER);
@@ -598,6 +649,9 @@ function HandleSync($backend, $protocolversion, $devid) {
                 $encoder->content(1);
                 $encoder->endTag();
 
+                //check the mimesupport because we need it for advanced emails
+                $mimesupport = isset($collection['mimesupport']) ? $collection['mimesupport'] : 0;
+
                 // Output server IDs for new items we received from the PDA
                 if(isset($collection["clientids"]) || count($collection["fetchids"]) > 0) {
                     $encoder->startTag(SYNC_REPLIES);
@@ -615,7 +669,7 @@ function HandleSync($backend, $protocolversion, $devid) {
                         $encoder->endTag();
                     }
                     foreach($collection["fetchids"] as $id) {
-                        $data = $backend->Fetch($collection["collectionid"], $id);
+                        $data = $backend->Fetch($collection["collectionid"], $id, $mimesupport);
                         if($data !== false) {
                             $encoder->startTag(SYNC_FETCH);
                             $encoder->startTag(SYNC_SERVERENTRYID);
@@ -636,13 +690,7 @@ function HandleSync($backend, $protocolversion, $devid) {
                 }
 
                 if(isset($collection["getchanges"])) {
-                    // Use the state from the importer, as changes may have already happened
-                    $exporter = $backend->GetExporter($collection["collectionid"]);
-
-                    $filtertype = isset($collection["filtertype"]) ? $collection["filtertype"] : false;
-                    $exporter->Config($importer, $collection["class"], $filtertype, $collection["syncstate"], 0, $collection["truncation"]);
-
-                    $changecount = $exporter->GetChangeCount();
+                    // exporter already intialized
 
                     if($changecount > $collection["maxitems"]) {
                         $encoder->startTag(SYNC_MOREAVAILABLE, false, true);
@@ -663,8 +711,10 @@ function HandleSync($backend, $protocolversion, $devid) {
                             break;
                         $n++;
 
-                        if($n >= $collection["maxitems"])
+                        if($n >= $collection["maxitems"]) {
+                        	debugLog("Exported maxItems of messages: ". $collection["maxitems"] . " - more available");
                             break;
+                        }
 
                     }
                     $encoder->endTag();
@@ -822,6 +872,7 @@ function HandleGetAttachment($backend, $protocolversion) {
 
 function HandlePing($backend, $devid) {
     global $zpushdtd, $input, $output;
+    global $user, $auth_pw;
     $timeout = 5;
 
     debugLog("Ping received");
@@ -848,6 +899,9 @@ function HandlePing($backend, $devid) {
         }
 
         if($decoder->getElementStartTag(SYNC_PING_FOLDERS)) {
+            // avoid ping init if not necessary
+            $saved_collections = $collections;
+
             $collections = array();
 
             while($decoder->getElementStartTag(SYNC_PING_FOLDER)) {
@@ -864,14 +918,27 @@ function HandlePing($backend, $devid) {
 
                 $decoder->getElementEndTag();
 
+                // initialize empty state
+                $collection["state"] = "";
+
+                // try to find old state in saved states
+                foreach ($saved_collections as $saved_col) {
+                    if ($saved_col["serverid"] == $collection["serverid"] && $saved_col["class"] == $collection["class"]) {
+                        $collection["state"] = $saved_col["state"];
+                        debugLog("reusing saved state for ". $collection["class"]);
+                        break;
+                    }
+                }
+
+                if ($collection["state"] == "")
+                    debugLog("empty state for ". $collection["class"]);
+
                 // Create start state for this collection
                 $exporter = $backend->GetExporter($collection["serverid"]);
-                $state = "";
                 $importer = false;
-                $exporter->Config($importer, false, false, $state, BACKEND_DISCARD_DATA, 0);
+                $exporter->Config($importer, false, false, $collection["state"], BACKEND_DISCARD_DATA, 0);
                 while(is_array($exporter->Synchronize()));
-                $state = $exporter->GetState();
-                $collection["state"] = $state;
+                $collection["state"] = $exporter->GetState();
                 array_push($collections, $collection);
             }
 
@@ -889,6 +956,16 @@ function HandlePing($backend, $devid) {
     debugLog("Waiting for changes... (lifetime $lifetime)");
     // Wait for something to happen
     for($n=0;$n<$lifetime / $timeout; $n++ ) {
+        //check the remote wipe status
+        if (PROVISIONING === true) {
+	        $rwstatus = $backend->getDeviceRWStatus($user, $auth_pw, $devid);
+	        if ($rwstatus == SYNC_PROVISION_RWSTATUS_PENDING || $rwstatus == SYNC_PROVISION_RWSTATUS_WIPED) {
+	            //return 7 because it forces folder sync
+	            $pingstatus = 7;
+	            break;
+	        }
+        }
+
         if(count($collections) == 0) {
             $error = 1;
             break;
@@ -940,6 +1017,8 @@ function HandlePing($backend, $devid) {
         $encoder->startTag(SYNC_PING_STATUS);
         if(isset($error))
             $encoder->content(3);
+        elseif (isset($pingstatus))
+            $encoder->content($pingstatus);
         else
             $encoder->content(count($changes) > 0 ? 2 : 1);
         $encoder->endTag();
@@ -1052,27 +1131,30 @@ function HandleFolderCreate($backend, $protocolversion) {
             return false;
     }
 
-    // Parent
-    $parentid = false;
-    if($decoder->getElementStartTag(SYNC_FOLDERHIERARCHY_PARENTID)) {
-        $parentid = $decoder->getElementContent();
-        if(!$decoder->getElementEndTag())
-            return false;
-    }
+    // when creating or updating more information is necessary
+    if (!$delete) {
+	    // Parent
+	    $parentid = false;
+	    if($decoder->getElementStartTag(SYNC_FOLDERHIERARCHY_PARENTID)) {
+	        $parentid = $decoder->getElementContent();
+	        if(!$decoder->getElementEndTag())
+	            return false;
+	    }
 
-    // Displayname
-    if(!$decoder->getElementStartTag(SYNC_FOLDERHIERARCHY_DISPLAYNAME))
-        return false;
-    $displayname = $decoder->getElementContent();
-    if(!$decoder->getElementEndTag())
-        return false;
+	    // Displayname
+	    if(!$decoder->getElementStartTag(SYNC_FOLDERHIERARCHY_DISPLAYNAME))
+	        return false;
+	    $displayname = $decoder->getElementContent();
+	    if(!$decoder->getElementEndTag())
+	        return false;
 
-    // Type
-    $type = false;
-    if($decoder->getElementStartTag(SYNC_FOLDERHIERARCHY_TYPE)) {
-        $type = $decoder->getElementContent();
-        if(!$decoder->getElementEndTag())
-            return false;
+	    // Type
+	    $type = false;
+	    if($decoder->getElementStartTag(SYNC_FOLDERHIERARCHY_TYPE)) {
+	        $type = $decoder->getElementContent();
+	        if(!$decoder->getElementEndTag())
+	            return false;
+	    }
     }
 
     if(!$decoder->getElementEndTag())
@@ -1083,15 +1165,28 @@ function HandleFolderCreate($backend, $protocolversion) {
     $syncstate = $statemachine->getSyncState($synckey);
     $newsynckey = $statemachine->getNewSyncKey($synckey);
 
+    // additional information about already seen folders
+    $seenfolders = unserialize($statemachine->getSyncState("s".$synckey));
+    if (!$seenfolders) $seenfolders = array();
+
     // Configure importer with last state
     $importer = $backend->GetHierarchyImporter();
     $importer->Config($syncstate);
 
-    // Send change
-    $serverid = $importer->ImportFolderChange($serverid, $parentid, $displayname, $type);
+    if (!$delete) {
+	    // Send change
+	    $serverid = $importer->ImportFolderChange($serverid, $parentid, $displayname, $type);
+    }
+    else {
+    	// delete folder
+    	$deletedstat = $importer->ImportFolderDeletion($serverid, 0);
+    }
 
     $encoder->startWBXML();
     if ($create) {
+    	// add folder id to the seen folders
+        $seenfolders[] = $serverid;
+
         $encoder->startTag(SYNC_FOLDERHIERARCHY_FOLDERCREATE);
         {
             {
@@ -1128,9 +1223,34 @@ function HandleFolderCreate($backend, $protocolversion) {
             $encoder->endTag();
         }
     }
+    elseif ($delete) {
+
+        $encoder->startTag(SYNC_FOLDERHIERARCHY_FOLDERDELETE);
+        {
+            {
+                $encoder->startTag(SYNC_FOLDERHIERARCHY_STATUS);
+                $encoder->content($deletedstat);
+                $encoder->endTag();
+
+                $encoder->startTag(SYNC_FOLDERHIERARCHY_SYNCKEY);
+                $encoder->content($newsynckey);
+                $encoder->endTag();
+            }
+            $encoder->endTag();
+        }
+
+        // remove folder from the folderflags array
+        if (($sid = array_search($serverid, $seenfolders)) !== false) {
+            unset($seenfolders[$sid]);
+            $seenfolders = array_values($seenfolders);
+            debugLog("deleted from seenfolders: ". $serverid);
+        }
+    }
+
     $encoder->endTag();
     // Save the sync state for the next time
     $statemachine->setSyncState($newsynckey, $importer->GetState());
+    $statemachine->setSyncState("s".$newsynckey, serialize($seenfolders));
 
     return true;
 }
@@ -1217,6 +1337,286 @@ function HandleFolderUpdate($backend, $protocolversion) {
     return HandleFolderCreate($backend, $protocolversion);
 }
 
+function HandleFolderDelete($backend, $protocolversion) {
+    return HandleFolderCreate($backend, $protocolversion);
+}
+
+function HandleProvision($backend, $devid, $protocolversion) {
+    global $user, $auth_pw, $policykey;
+
+    global $zpushdtd, $policies;
+    global $output, $input;
+
+    $status = SYNC_PROVISION_STATUS_SUCCESS;
+
+    $phase2 = true;
+
+    $decoder = new WBXMLDecoder($input, $zpushdtd);
+    $encoder = new WBXMLEncoder($output, $zpushdtd);
+
+    if(!$decoder->getElementStartTag(SYNC_PROVISION_PROVISION))
+        return false;
+
+    //handle android remote wipe.
+    if ($decoder->getElementStartTag(SYNC_PROVISION_REMOTEWIPE)) {
+        if(!$decoder->getElementStartTag(SYNC_PROVISION_STATUS))
+            return false;
+
+        $status = $decoder->getElementContent();
+
+        if(!$decoder->getElementEndTag())
+            return false;
+
+        if(!$decoder->getElementEndTag())
+            return false;
+    }
+
+    else {
+
+        if(!$decoder->getElementStartTag(SYNC_PROVISION_POLICIES))
+            return false;
+
+        if(!$decoder->getElementStartTag(SYNC_PROVISION_POLICY))
+            return false;
+
+        if(!$decoder->getElementStartTag(SYNC_PROVISION_POLICYTYPE))
+            return false;
+
+        $policytype = $decoder->getElementContent();
+        if ($policytype != 'MS-WAP-Provisioning-XML') {
+            $status = SYNC_PROVISION_STATUS_SERVERERROR;
+        }
+        if(!$decoder->getElementEndTag()) //policytype
+            return false;
+
+        if ($decoder->getElementStartTag(SYNC_PROVISION_POLICYKEY)) {
+            $devpolicykey = $decoder->getElementContent();
+
+            if(!$decoder->getElementEndTag())
+                return false;
+
+            if(!$decoder->getElementStartTag(SYNC_PROVISION_STATUS))
+                return false;
+
+            $status = $decoder->getElementContent();
+            //do status handling
+            $status = SYNC_PROVISION_STATUS_SUCCESS;
+
+            if(!$decoder->getElementEndTag())
+                return false;
+
+            $phase2 = false;
+        }
+
+        if(!$decoder->getElementEndTag()) //policy
+            return false;
+
+        if(!$decoder->getElementEndTag()) //policies
+            return false;
+
+        if ($decoder->getElementStartTag(SYNC_PROVISION_REMOTEWIPE)) {
+            if(!$decoder->getElementStartTag(SYNC_PROVISION_STATUS))
+                return false;
+
+            $status = $decoder->getElementContent();
+
+            if(!$decoder->getElementEndTag())
+                return false;
+
+            if(!$decoder->getElementEndTag())
+                return false;
+        }
+    }
+    if(!$decoder->getElementEndTag()) //provision
+        return false;
+
+    $encoder->StartWBXML();
+
+    //set the new final policy key in the backend
+    if (!$phase2) {
+        $policykey = $backend->generatePolicyKey();
+        $backend->setPolicyKey($policykey, $devid);
+    }
+
+    $encoder->startTag(SYNC_PROVISION_PROVISION);
+    {
+        $encoder->startTag(SYNC_PROVISION_STATUS);
+            $encoder->content($status);
+        $encoder->endTag();
+
+        $encoder->startTag(SYNC_PROVISION_POLICIES);
+            $encoder->startTag(SYNC_PROVISION_POLICY);
+
+            $encoder->startTag(SYNC_PROVISION_POLICYTYPE);
+                   $encoder->content($policytype);
+            $encoder->endTag();
+
+            $encoder->startTag(SYNC_PROVISION_STATUS);
+                $encoder->content($status);
+            $encoder->endTag();
+
+            $encoder->startTag(SYNC_PROVISION_POLICYKEY);
+                   $encoder->content($policykey);
+            $encoder->endTag();
+
+            if ($phase2) {
+                $encoder->startTag(SYNC_PROVISION_DATA);
+                if ($policytype == 'MS-WAP-Provisioning-XML') {
+                    $encoder->content('<wap-provisioningdoc><characteristic type="SecurityPolicy"><parm name="4131" value="1"/><parm name="4133" value="1"/></characteristic></wap-provisioningdoc>');
+                }
+                else {
+                    debugLog("Wrong policy type");
+                    return false;
+                }
+
+                $encoder->endTag();//data
+            }
+            $encoder->endTag();//policy
+        $encoder->endTag(); //policies
+    }
+    $rwstatus = $backend->getDeviceRWStatus($user, $auth_pw, $devid);
+
+
+    //wipe data if status is pending or wiped
+    if ($rwstatus == SYNC_PROVISION_RWSTATUS_PENDING || $rwstatus == SYNC_PROVISION_RWSTATUS_WIPED) {
+        $encoder->startTag(SYNC_PROVISION_REMOTEWIPE, false, true);
+        $backend->setDeviceRWStatus($user, $auth_pw, $devid, SYNC_PROVISION_RWSTATUS_WIPED);
+        //$rwstatus = SYNC_PROVISION_RWSTATUS_WIPED;
+    }
+
+    $encoder->endTag();//provision
+
+    return true;
+}
+
+
+function HandleSearch($backend, $devid, $protocolversion) {
+    global $zpushdtd;
+    global $input, $output;
+
+    $searchrange = '0';
+
+    $decoder = new WBXMLDecoder($input, $zpushdtd);
+    $encoder = new WBXMLEncoder($output, $zpushdtd);
+
+    if(!$decoder->getElementStartTag(SYNC_SEARCH_SEARCH))
+        return false;
+
+    if(!$decoder->getElementStartTag(SYNC_SEARCH_STORE))
+        return false;
+
+    if(!$decoder->getElementStartTag(SYNC_SEARCH_NAME))
+        return false;
+    $searchname = $decoder->getElementContent();
+    if(!$decoder->getElementEndTag())
+        return false;
+
+    if(!$decoder->getElementStartTag(SYNC_SEARCH_QUERY))
+        return false;
+    $searchquery = $decoder->getElementContent();
+    if(!$decoder->getElementEndTag())
+        return false;
+
+    if($decoder->getElementStartTag(SYNC_SEARCH_OPTIONS)) {
+        while(1) {
+            if($decoder->getElementStartTag(SYNC_SEARCH_RANGE)) {
+                $searchrange = $decoder->getElementContent();
+                if(!$decoder->getElementEndTag())
+                    return false;
+                }
+                $e = $decoder->peek();
+                if($e[EN_TYPE] == EN_TYPE_ENDTAG) {
+                    $decoder->getElementEndTag();
+                    break;
+                }
+            }
+        //if(!$decoder->getElementEndTag())
+            //return false;
+    }
+    if(!$decoder->getElementEndTag()) //store
+        return false;
+
+    if(!$decoder->getElementEndTag()) //search
+        return false;
+
+
+    if (strtoupper($searchname) != "GAL") {
+        debugLog("Searchtype $searchname is not supported");
+        return false;
+    }
+    //get search results from backend
+    $rows = $backend->getSearchResults($searchquery, $searchrange);
+
+    $encoder->startWBXML();
+
+    $encoder->startTag(SYNC_SEARCH_SEARCH);
+
+        $encoder->startTag(SYNC_SEARCH_STATUS);
+        $encoder->content(1);
+        $encoder->endTag();
+
+        $encoder->startTag(SYNC_SEARCH_RESPONSE);
+            $encoder->startTag(SYNC_SEARCH_STORE);
+
+                $encoder->startTag(SYNC_SEARCH_STATUS);
+                $encoder->content(1);
+                $encoder->endTag();
+
+                if (is_array($rows) && !empty($rows)) {
+                    $searchrange = $rows['range'];
+                    unset($rows['range']);
+                    foreach ($rows as $u) {
+                        $encoder->startTag(SYNC_SEARCH_RESULT);
+                            $encoder->startTag(SYNC_SEARCH_PROPERTIES);
+
+                                $encoder->startTag(SYNC_GAL_DISPLAYNAME);
+                                $encoder->content($u["fullname"]);
+                                $encoder->endTag();
+
+                                $encoder->startTag(SYNC_GAL_PHONE);
+                                $encoder->content($u["businessphone"]);
+                                $encoder->endTag();
+
+                                $encoder->startTag(SYNC_GAL_ALIAS);
+                                $encoder->content($u["username"]);
+                                $encoder->endTag();
+
+                                //it's not possible not get first and last name of an user
+                                //from the gab and user functions, so we just set fullname
+                                //to lastname and leave firstname empty because nokia needs
+                                //first and lastname in order to display the search result
+                                $encoder->startTag(SYNC_GAL_FIRSTNAME);
+                                $encoder->content("");
+                                $encoder->endTag();
+
+                                $encoder->startTag(SYNC_GAL_LASTNAME);
+                                $encoder->content($u["fullname"]);
+                                $encoder->endTag();
+
+                                $encoder->startTag(SYNC_GAL_EMAILADDRESS);
+                                $encoder->content($u["emailaddress"]);
+                                $encoder->endTag();
+
+                            $encoder->endTag();//result
+                        $encoder->endTag();//properties
+                    }
+                    $encoder->startTag(SYNC_SEARCH_RANGE);
+                    $encoder->content($searchrange);
+                    $encoder->endTag();
+
+                    $encoder->startTag(SYNC_SEARCH_TOTAL);
+                    $encoder->content(count($rows));
+                    $encoder->endTag();
+                }
+
+            $encoder->endTag();//store
+        $encoder->endTag();//response
+    $encoder->endTag();//search
+
+
+    return true;
+}
+
 function HandleRequest($backend, $cmd, $devid, $protocolversion) {
     switch($cmd) {
         case 'Sync':
@@ -1273,6 +1673,13 @@ function HandleRequest($backend, $cmd, $devid, $protocolversion) {
         case 'Ping': // Used for http-based notifications (pushmail)
             $status = HandlePing($backend, $devid, $protocolversion);
             break;
+        case 'Provision':
+            $status = (PROVISIONING === true) ? HandleProvision($backend, $devid, $protocolversion) : false;
+            break;
+        case 'Search':
+            $status = HandleSearch($backend, $devid, $protocolversion);
+            break;
+
         default:
             debugLog("unknown command - not implemented");
             $status = false;
